*                                    WARNING

* The Government of Canada (Statistics Canada) is the owner of all intellectual
* property rights (including copyright) in this software.  Subject to the terms below,
* you are granted a non-exclusive and non-transferable licence to use this software.
*
* This software is provided "as-is", and the owner makes no warranty, either express
* or implied, including but not limited to, warranties of merchantability and fitness
* for any particular purpose.  In no event will the owner be liable for any indirect,
* special, consequential or other similar damages.  This agreement will terminate
* automatically without notice to you if you fail to comply with any term of this
* agreement.;

/*******************************************************************/
/* Date: April 2004                                                */
/*******************************************************************/

/*********************************************************************************/
/***                                                                           ***/
/***                            MACROE_V30.SAS                                 ***/
/***                            (Version 3.0.1)                                ***/
/***                                                                           ***/
/*** This program calculates variance estimates using the bootstrap weights    ***/
/*** for different types of estimators.  Using SAS Macros, this program can    ***/
/*** calculate variance estimates for totals, ratios and differences between   ***/
/*** ratios.  It can also calculate variance estimates for the parameters      ***/
/*** of a linear regression or logistic regression. This program can also be   ***/
/*** customized for other types of analyses.                                   ***/
/***                                                                           ***/
/*** This program contains the macros that are necessary to use the 	       ***/
/*** BOOTVARE_V30.SAS program.                                                 ***/
/***                                                                           ***/
/*** This program is automatically called by BOOTVARE_V30.SAS and NO MODIFICA- ***/
/*** TIONS SHOULD BE MADE BY THE USER (except for specific cases mentioned in  ***/
/*** BOOTVARE_V30.SAS)                                                         ***/
/***                                                                           ***/
/*********************************************************************************/

options ps=64 ls=120 nonotes;


*********************************************************************************
*** Section 1: Declaration of the Macro Variables                             ***
*********************************************************************************;

/*  Verification if breakdown variable(s) */ 

%let by=;
%let cla_tmp="&classes";

data _NULL_;
if substr(&cla_tmp,1,1)='.' then do;
                   call symput ('by' ,'*');
                   call symput ('classes' ,'');
                   call symput ('number',0);
				   end;
run;

* TO OBTAIN THE MARGINALS IN THE OUTPUT;


&by  %let cla_tmp="&classes"||' #';

&by  data _NULL_;
&by  do i=1 to 10;
&by  	call symput ('cla'||left(trim(i)),' ');
&by  end;
&by  init=1; &by  i=1; &by  fin=1; &by  stop=' ';
&by  do until (stop='#');
&by  	do until (substr(&cla_tmp,i,1)='');
&by  		call symput ('cla'||left(trim(init)),substr(&cla_tmp,fin,i-fin+1));
&by  		call symput ('number',init);
&by  		i=i+1;
&by  	end;
&by  	fin=i;
&by  	stop=substr(&cla_tmp,i+1,1);
&by  	init=init+1;
&by  end;
&by  run;


* VARIABLE SPECIFIC TO EACH VERSION OF SAS ;

data _NULL_;
if &sysver >= 8 then call symput ('inter' ,'intercept');
else call symput ('inter' ,'intercep');
run;

* VARIABLES FOR INDEX(next section):   ;

%let indx=(id=(&ident blank)); 


***************************************************************************;
*  SECTION 2: READING IN THE MAIN FILE AND MERGING TO THE WEIGHTS         *;
***************************************************************************;

OPTIONS notes;

data Mfile (index=&indx);
   set &Mfile ;
   blank=.;
run;

/* The next step reads the bootstrap weights and standardize the name of the bootatrap weigths varaibles */
/* FWGT is the same weight as on the analysis file */

OPTIONS nonotes;

data bsamp (index=&indx);
  set &bsamp;
   blank=.;
         if &bsw.1 ne '' then do; 
                           call symput ('bsw_frst' ,'&bsw.1');
                           call symput ('bsw_last' ,'&bsw.&b');
						end;

         else if &bsw.001 ne '' then do; 
                           call symput ('bsw_frst' ,'&bsw.001');
                           call symput ('bsw_last' ,'&bsw.%sysfunc(putn(&b,z3.))');
						end;
         else if &bsw.0001 ne '' then do; 
                           call symput ('bsw_frst' ,'&bsw.0001');
                           call symput ('bsw_last' ,'&bsw.%sysfunc(putn(&b,z4.))');
						end;   
run;

OPTIONS notes;

/* Merging the main file and the bootstrap weights */

     data bs_data ;
       merge Mfile (in=in1) bsamp (keep=&fwgt &ident &bsw_frst-&bsw_last);
       by &ident;
       if in1;
	   drop blank;
       rename &bsw_frst-&bsw_last=bsw1-bsw&b; 
     run;


&by  proc sort data=bs_data;
&by     by &classes;
&by  run;


* RESULTS FILES :     ;

data alltots allrats diffrat bs_reg bs_reglg ;
      set _NULL_;
run;

%let result= alltots allrats diffrat bs_reg bs_reglg;


/*********************************************************************************/
/*** Section 3: Declaration of the macros                                      ***/
/*********************************************************************************/

%let printtot=0;
%let printrat=0;
%let printdif=0;
%let printreg=0;
%let printlog=0;

%global dep1 dep2;

*************************************;

%macro total(var);

*************************************;

proc means data=bs_data noprint;
  var &fwgt bsw1-bsw&b;
  weight &var;
&by  class &classes;
  output out=ytot
         sum=yhat ybs1-ybs&b;
run;


proc means data=Mfile noprint;
  var &var;
  where &var>0;
&by  class &classes;
  output out=n n=n;
run;

data ytot;
	merge ytot n (drop=_type_ _freq_);
&by by &classes;
run;

     data est;
		  set ytot;
		  length var $ 8;
		  length type $ 8;
		  Estimate=yhat;
		  bs_var=((&b-1)*(var(of ybs1-ybs&b)))/&b;
		  bs_sd=sqrt(bs_var);
		  bs_cv=round((bs_sd/yhat)*100,.01);
		  cil95=yhat-1.96*bs_sd;
		  ciu95=yhat+1.96*bs_sd;
		  var="&var";
		  type="Total";
		  drop ybs1-ybs&b _type_ _freq_;
&by		  drop &cla1 &cla2 &cla3 &cla4 &cla5 &cla6 &cla7 &cla8 &cla9 &cla10;

&by  			%do k=1 %to &number; 
&by  			if &&cla&k ne ' ' then cla&k=put(&&cla&k,best8.);
&by  			%end;
run;

data alltots;
	set alltots est;
run;

%let printtot=1;

proc datasets library=work;
	delete ytot est;
run;

%mend total;


*******************************************;

%macro ratio(var1,var2,);

*******************************************;

proc means data=bs_data noprint;
  var &fwgt bsw1-bsw&b;
  weight &var1;
&by  class &classes;
  output out=ytot
         sum=yhat ybs1-ybs&b;
run;

proc means data=bs_data noprint;
  var &fwgt bsw1-bsw&b;
  weight &var2;
  &by  class &classes;
  output out=xtot
  sum=xhat xbs1-xbs&b;
run;

proc means data=Mfile noprint;
  var &var1;
   where &var1>0;
&by  class &classes;
  output out=n n=n1;
run;

data ytot;
	merge ytot n (drop=_type_ _freq_);
&by by  &classes;
run;


data est;
  merge ytot xtot;
  array ybs{&b};
  array xbs{&b};
  array rbs{&b};
  length var1 $ 8;
  length var2 $ 8;
  length type $ 8;
  Estimate=((yhat/xhat));
  do i=1 to &b;
    rbs{i}=((ybs{i}/xbs{i}));
  end;
  bs_var=((&b-1)*(var(of rbs1-rbs&b)))/&b;
  bs_sd=sqrt(bs_var);
  bs_cv=round((bs_sd/Estimate)*100,.01);
  cil95=Estimate-1.96*bs_sd;
  ciu95=Estimate+1.96*bs_sd;
  var1="&var1";
  var2="&var2";
  type="Ratio";
  drop ybs1-ybs&b xbs1-xbs&b rbs1-rbs&b xhat yhat i _type_ _freq_;
&by		  drop &cla1 &cla2 &cla3 &cla4 &cla5 &cla6 &cla7 &cla8 &cla9 &cla10;

&by  			%do k=1 %to &number; 
&by  			if &&cla&k ne ' ' then cla&k=put(&&cla&k,best8.);
&by  			%end;
run;

data allrats;
	set allrats est;
run;

%let printrat=1;

proc datasets library=work;
    delete ytot xtot est;
run;

%mend ratio;


********************************************************;

%macro diff_rat(var1,var2,var3,var4);

********************************************************;

proc means data=bs_data noprint;
  var &fwgt bsw1-bsw&b;
  weight &var1;
  &by  class &classes;
  output out=ytot
  sum=yhat ybs1-ybs&b;
run;

proc means data=bs_data noprint;
  var &fwgt bsw1-bsw&b;
  weight &var2;
  &by  class &classes;
  output out=xtot
  sum=xhat xbs1-xbs&b;
run;

proc means data=bs_data noprint;
  var &fwgt bsw1-bsw&b;
  weight &var3;
  &by  class &classes;
  output out=yytot
  sum=yyhat yybs1-yybs&b;
run;

proc means data=bs_data noprint;
  var &fwgt bsw1-bsw&b;
  weight &var4;
  &by  class &classes;
  output out=xxtot
  sum=xxhat xxbs1-xxbs&b;
run;

proc means data=Mfile noprint;
  var &var1;
  where &var1>0;
&by  class &classes;
  output out=n1 n=n1;
run;

proc means data=Mfile noprint;
  var &var3;
  where &var3>0;
&by  class &classes;
  output out=n3 n=n3;
run;

data ytot;
	merge ytot n1 (drop=_type_ _freq_) n3 (drop=_type_ _freq_);
&by by &classes;
run;

data est;
  merge ytot xtot yytot xxtot;
  array ybs{&b};
  array xbs{&b};
  array yybs{&b};
  array xxbs{&b};
  array drbs{&b};
  length var1 $ 8;
  length var2 $ 8;
  length var3 $ 8;
  length var4 $ 8;
  length type $ 10;
  Estimate=(((yhat/xhat)-(yyhat/xxhat)));
  do i=1 to &b;
    drbs{i}=(((ybs{i}/xbs{i})-(yybs{i}/xxbs{i})));
  end;
  bs_var=(((&b-1)*(var(of drbs1-drbs&b)))/&b);
  bs_sd=sqrt(bs_var);
  bs_cv=abs(round((bs_sd/Estimate)*100,.01));
  cil95=Estimate-1.96*bs_sd;
  ciu95=Estimate+1.96*bs_sd;
  var1="&var1";
  var2="&var2";
  var3="&var3";
  var4="&var4";
  type="Dif_Rat";
  drop ybs1-ybs&b xbs1-xbs&b yybs1-yybs&b xxbs1-xxbs&b drbs1-drbs&b
       xhat yhat xxhat yyhat i _type_ _freq_;
&by		  drop &cla1 &cla2 &cla3 &cla4 &cla5 &cla6 &cla7 &cla8 &cla9 &cla10;

&by  			%do k=1 %to &number; 
&by  			if &&cla&k ne ' ' then cla&k=put(&&cla&k,best8.);
&by  			%end;
run;

data diffrat;
	set diffrat est;
run;


%let printdif=1;

proc datasets library=work;
   delete ytot xtot yytot xxtot est;
run;

%mend diff_rat;


*********************************************;

%macro regress(yvar,xvar);

*********************************************;

proc reg data=bs_data outest=orig(keep=&classes &inter &xvar) noprint;
  model &yvar=&xvar;
  weight &fwgt;
&by  by &classes;
run;

proc transpose data=orig out=origest(drop=_label_) prefix=beta name=param;
  var &inter &xvar;
&by  by &classes;
run;

data _NULL_; 
    L=int((&b/10)+0.999);
    call symput ('L' , trim(left(L))); 
run;

OPTIONS nonotes;

%let j_dep=1;

%do k=1 %to 10;
  %let j=%eval(1+((&k-1)*&L));
  %let kL=%eval(&k*&L);

data _NULL_;
if (&b - &kL) >0  then do;	
                          k=&k;
                          j=1+((&k-1)*&L);
                          kL=&k*&L;
                       end;
                  else do; 	
                          k=10; 
                          j=&j_dep;
                          kL=&b;
                       end;

call symput ('k' , trim(left(k)));
call symput ('j' , trim(left(j)));
call symput ('kL' , trim(left(kL)));
run;

data poids (keep = bsw&j-bsw&kL &yvar &xvar &classes);  
  set bs_data;
run;

  %do i=&j %to &kL;

     %put Regression &i completed;

   %let j_dep=%eval(&kl+1);

   proc reg data=poids outest=betas(keep=&classes &inter &xvar) noprint;
     model &yvar=&xvar;
     weight bsw&i;
&by  by &classes;
   run;

   proc transpose data=betas out=betat prefix=best name=param;
     var &inter &xvar;
&by  by &classes;
   run;

   data betat;
     set betat;
     drop _label_;
     rename best1=best&i;
	if &i=1 then test=1;
	else test=test+1;
   run;

   %if (&i =1) %then %do;

     data bsbeta;
       set betat;
     run;

   %end;
   %else %do;

     data bsbeta;
       merge bsbeta betat;
&by    by &classes;
     run;

   %end;
  %end;
%end;

OPTIONS notes;

data est;
  merge origest bsbeta;
  rename beta1=beta;
  bs_var=((&b-1)*(var(of best1-best&b)))/&b;
  bs_sd=sqrt(bs_var);
  bs_cv=abs(round((bs_sd/beta1)*100,.01));
  cil95=beta1-1.96*bs_sd;
  ciu95=beta1+1.96*bs_sd;
  ydep="&yvar";
  drop best1-best&b;
run;

data bs_reg;
    set bs_reg est;
run;

%let printreg=1;
%let dep1=&yvar;

proc datasets library=work;
    delete betas betat bsbeta origest;
run;


%mend regress;

*********************************************;

%macro logreg(yvar,xvar);

*********************************************;

proc logistic data=bs_data outest=orig(keep=&classes &inter &xvar) descending noprint;
  model &yvar=&xvar;
  &by by &classes;
  weight &fwgt;
run;

proc transpose data=orig out=origest prefix=beta name=param;
  var &inter &xvar;
  &by by &classes;
run;


data _NULL_; 
    L=int((&b/10)+0.999);
    call symput ('L' , trim(left(L))); 
run;

OPTIONS nonotes;

%let j_dep=1;

%do k=1 %to 10;
  %let j=%eval(1+((&k-1)*&L));
  %let kL=%eval(&k*&L);

data _NULL_;
if (&b - &kL) >0  then do;	
                          k=&k;
                          j=1+((&k-1)*&L);
                          kL=&k*&L;
                       end;
                  else do; 	
                          k=10; 
                          j=&j_dep;
                          kL=&b;
                       end;

call symput ('k' , trim(left(k)));
call symput ('j' , trim(left(j)));
call symput ('kL' , trim(left(kL)));
run;


data poids (keep = bsw&j-bsw&kL &yvar &xvar &classes);  
  set bs_data;
run;

  %do i=&j %to &kL;

     %put Logistic regression &i completed;

     %let j_dep=%eval(&kl+1);

    proc logistic data=poids outest=betas (keep=&classes &inter &xvar) noprint descending;
      model &yvar=&xvar;
      &by by &classes;
      weight bsw&i;
    run;



    proc transpose data=betas out=betat prefix=best name=param;
      var &inter &xvar;
      &by by &classes;
    run;

    data betat;
      set betat;
      rename best1=best&i;
    run;

    %if (&i =1) %then %do;

      data bsbeta;
        set betat;
      run;

    %end;
    %else %do;

      data bsbeta;
        merge bsbeta betat;
        &by by &classes;
      run;

    %end;
  %end;
%end;

OPTIONS notes;


data est;
  merge origest bsbeta;
  rename beta1=beta;
  bs_var=((&b-1)*(var(of best1-best&b)))/&b;
  bs_sd=sqrt(bs_var);
  bs_cv=abs(round((bs_sd/beta1)*100,.01));
  wald=(beta1/bs_sd)*(beta1/bs_sd);
  pvalue=1-probchi(wald,1);
  lo95=beta1-1.96*bs_sd;
  hi95=beta1+1.96*bs_sd;
  odds=exp(beta1);
  cil95=exp(lo95);
  ciu95=exp(hi95);
  ydep="&yvar";
  drop best1-best&b;
run;

data bs_reglg; 
    set bs_reglg est;
run;

%let printlog=1;
%let dep2=&yvar;

proc datasets library=work;
    delete betas betat bsbeta origest;
run;


%mend logreg;



****************;

%macro prnttot;

****************;
%if &printtot=1 %then %do;

&by  data alltots;
&by  	set alltots;
&by     ind1=3; &by  ind2=3; &by  ind3=3; &by  ind4=3;&by   ind5=3; &by  ind6=3; &by  ind7=3;&by   ind8=3;&by   ind9=3;&by   ind10=3;  
&by      %do i=1 %to &number;
&by  	     &&cla&i=cla&i;
&by  	     if &&cla&i=" " then &&cla&i="     All";
&by  	     if &&cla&i="     All" then ind&i=1;  &by  else ind&i=2;
&by  	%end;
&by  	run;

&by   proc sort data=alltots;
&by     by ind1 ind2 ind3 ind4 ind5 ind6 ind7 ind8 ind9 ind10  &classes;
&by   run;

/*******************************************/
/* Prints the results of the total macro   */
/*******************************************/

proc print data=alltots;
    	 title "Variance Estimation for Totals";
    	 title2 "using &B bootstrap replicates";
	     title3 ;
	 var &classes type var n Estimate bs_sd bs_cv cil95 ciu95;
  	 format Estimate bs_sd cil95 ciu95 11.2;
run;

%end;

        /*****************************************************************/
        /*** Where:                                                    ***/
        /*** type         : estimate type (total )                     ***/
        /*** var          : variable used to calculate the estimate    ***/
        /*** n            : sample size for the estimate               ***/
        /*** Estimate     : parameter estimate                         ***/
        /*** bs_sd        : standard deviation                         ***/
        /*** bs_cv        : coefficient of variation                   ***/
        /*** cil95        : lower bound of the 95% confidence interval ***/
        /*** ciu95        : upper bound of the 95% confidence interval ***/
        /*****************************************************************/


%mend prnttot;


****************;

%macro prntrat;

****************;
%if &printrat=1 %then %do;

&by  data allrats;
&by  	set allrats;
&by     ind1=3; &by  ind2=3; &by  ind3=3; &by  ind4=3;&by   ind5=3; &by  ind6=3; &by  ind7=3;&by   ind8=3;&by   ind9=3;&by   ind10=3;  
&by      %do i=1 %to &number;
&by  	     &&cla&i=cla&i;
&by  	     if &&cla&i=" " then &&cla&i="     All";
&by  	     if &&cla&i="     All" then ind&i=1;  &by  else ind&i=2;
&by  	%end;
&by  	run;

&by   proc sort data=allrats;
&by     by ind1 ind2 ind3 ind4 ind5 ind6 ind7 ind8 ind9 ind10  &classes;
&by   run;

/*******************************************/
/* Prints the results of the ratio macro   */
/*******************************************/

proc print data=allrats;
    	 title "Variance Estimation for Ratios";
    	 title2 "using &B bootstrap replicates ";
	     title3 ;
	 var &classes type var1 var2 n1 Estimate bs_sd bs_cv cil95 ciu95;
     format  bs_sd cil95 ciu95 Estimate 11.4;
run;

%end;

        /*****************************************************************/
        /*** Where:                                                    ***/
        /*** type         : estimate type (ratio)                      ***/
        /*** var1 et var2 : variables used to calculate the estimates. ***/
        /*** n1           : sample size for the numerator (var1)       ***/
        /*** Estimate     : parameter estimate                         ***/
        /*** bs_sd        : standard deviation                         ***/
        /*** bs_cv        : coefficient of variation                   ***/
        /*** cil95        : lower bound of the 95% confidence interval ***/
        /*** ciu95        : upper bound of the 95% confidence interval ***/
        /*****************************************************************/


%mend prntrat;

****************;

%macro prntdiff;

****************;
%if &printdif=1 %then %do;

&by  data diffrat;
&by  	set diffrat;
&by     ind1=3; &by  ind2=3; &by  ind3=3; &by  ind4=3;&by   ind5=3; &by  ind6=3; &by  ind7=3;&by   ind8=3;&by   ind9=3;&by   ind10=3;  
&by      %do i=1 %to &number;
&by  	     &&cla&i=cla&i;
&by  	     if &&cla&i=" " then &&cla&i="    Tous";
&by  	     if &&cla&i="    Tous" then ind&i=1;  &by  else ind&i=2;
&by  	%end;
&by  	run;

&by   proc sort data=diffrat;
&by     by ind1 ind2 ind3 ind4 ind5 ind6 ind7 ind8 ind9 ind10  &classes;
&by   run;


/**********************************************/
/* Prints the results of the diff_rat macro   */
/**********************************************/

proc print data=diffrat;
  	 title "Variance Estimation for Differences between Ratios";
	 title2 "using &B bootstrap replicates ";
	 title3 ;
     var &classes type var1 var2 var3 var4 n1 n3 Estimate bs_sd bs_cv cil95 ciu95;
     format  bs_sd cil95 ciu95 Estimate 11.4;
   run;
%end;

        /*****************************************************************/
        /*** Where:                                                    ***/
        /*** type         : estimate type (ratio difference)           ***/
        /*** var1, var2,                                               ***/
        /*** var3 and var4: variables used to calculate the estimates. ***/
        /*** n1           : sample size for the first numerator (var1) ***/
        /*** n3           : sample size for the second numerator (var2)***/
        /*** Estimate     : estimate                                   ***/
        /*** bs_sd        : standard deviation                         ***/
        /*** bs_cv        : coefficient of variation                   ***/
        /*** cil95        : lower bound of the 95% confidence interval ***/
        /*** ciu95        : upper bound of the 95% confidence interval ***/
        /*****************************************************************/


%mend prntdiff;

****************;

%macro prntreg;

****************;
%if &printreg=1 %then %do;


/******************************************************/
/* Prints the results of the regress macro            */
/******************************************************/

   proc print data=bs_reg;
     title "Variance Estimation for a Regression ";
     title2 "Dependent variable: &dep1";
     title3 "using &B bootstrap replicates ";
     var &classes param beta bs_var bs_sd bs_cv cil95 ciu95;
   run;
%end;

        /**************************************************************************/
        /***  Where:                                                            ***/
        /***   param      : parameter to estimate                               ***/
        /***   beta       : parameter estimate                                  ***/
        /***   bsvar      : variance of the parameter estimate                  ***/
        /***   bs_sd      : standard deviation of the parameter estimate        ***/
        /***   bs_cv      : coefficient of variation for the parameter estimate ***/
        /***   cil95      : lower bound of the 95% confidence interval          ***/
        /***   ciu95      : upper bound of the 95% confidence interval          ***/
        /**************************************************************************/


%mend prntreg;

****************;

%macro prntlog;

****************;
%if &printlog=1 %then %do;

/********************************************/
/*Prints the results of the logreg macro    */
/********************************************/

   proc print data=bs_reglg;
     title "Variance Estimation for a Regression ";
     title2 "Dependent variable: &dep2";
     title3 "using &B bootstrap replicates ";
     var  &classes param beta odds wald pvalue bs_var bs_sd bs_cv cil95 ciu95;
	 run;
%end;

        /**************************************************************************/
        /***  Where:                                                            ***/
        /***   param      : parameter to estimate                               ***/
        /***   beta       : parameter estimate                                  ***/
        /***   odds       : odds ratio                                          ***/
        /***   wald       : Wald's statistic                                    ***/
        /***   pvalue     : p-value of Wald's statistic                         ***/
        /***   bsvar      : variance of the parameter estimate                  ***/
        /***   bs_sd      : standard deviation of the parameter estimate        ***/
        /***   bs_cv      : coefficient of variation for the parameter estimate ***/
        /***   cil95      : lower bound of the 95% confidence interval          ***/
        /***   ciu95      : upper bound of the 95% confidence interval          ***/
        /**************************************************************************/


%mend prntlog;


****************;

%macro output;

****************;

	%prnttot;
	%prntrat;
	%prntdiff;
	%prntreg;
	%prntlog;

	proc datasets library=work;
	      delete Mfile bsamp version tmp;
	run;
	quit;

%mend output;



/* End of MACROE_V30.SAS SAS program */
