***********************************************************
*                    STEP1.SAS                            *
*                                                         *
*        This program creates the SAS datafile            *
*        containing the necessary variables               *
*        for the BOOTVARE_V30.SAS program                 *
***********************************************************;


LIBNAME in1 'directory_to_save_file_in';


*** Creation of the SAS data file containing the variables and cases required for the analysis. Note
*** that this file should be as small as possible (containing only necessary variables and cases) in
*** order to reduce time and memory requirements especially if regression type analysis are to be done.  ;


data in1.Name_of_analysis_file;                 /* file to be used with BOOTVARE_V30.SAS */
    %let datafid= " name_and_location_of_source_file ";
    %include " name_and_location_of_layout ";


***  Creation of Dichotomous Variables                             ***
*** (examples are presented below using National Population Health ***
***  survry, cycle 3 variables)                                    ***;
            
   /* diabetes */
          if ccc8_1j=1 then diab=1;
          else diab=0;

   /* sex */
          if dhc8_sex=1 then males=1;
          else males=0;
          if dhc8_sex=2 then females=1;
          else females=0;
            
   /* diabetes*sex */
          mdiab = diab * males;      /* male diabetics   */
          fdiab = diab * females;    /* female diabetics */
            


   keep list of variables to keep;

          * It is recommended that only the necessary variables be kept    *
          * in order to reduce the runtime of BOOTVARE_V30.SAS.            *
          * IMPORTANT:  the identification variables and, if necessary,    *
          * the breakdown variable (ex: province, sex) must be kept. The   *
          * weight variable also must be kept if point estimates are       *
          * calculated at this step                                        *;
         
run;



*******************************************
* Calculation of point estimates          *
* Suggested, but not required...          *
*******************************************;


proc freq data=in1.Name_of_analysis_file;
     table variables of interest;
     weight weight_variable;
run;

proc logistic data=in1.Name_of_analysis_file ;
     model dependent_variable = independent variables ;
     weight weight_variable;
run;