* IMPORTANT NOTE: SPSS BOOTVAR VERSION 3.0 WAS TESTED AND WORKS WITH
*                               VERSION 11.5 (AND UNDER).  TESTS WERE CONDUCTED WITH
*                               VERSIONS 12 AND 13 OF SPSS AND PRELIMINARY RESULTS
*                               SHOWED MEMORY LIMITATION ISSUES.


*                                    WARNING

* The Government of Canada (Statistics Canada) is the owner of all intellectual
* property rights (including copyright) in this software.  Subject to the terms below,
* you are granted a non-exclusive and non-transferable licence to use this software.
*
* This software is provided "as-is", and the owner makes no warranty, either express
* or implied, including but not limited to, warranties of merchantability and fitness
* for any particular purpose.  In no event will the owner be liable for any indirect,
* special, consequential or other similar damages.  This agreement will terminate
* automatically without notice to you if you fail to comply with any term of this
* agreement.;

*******************************************************************
* Date: April 2002                                                         *
* Modified:  April 2004                                                   *
*******************************************************************.

**********************************************************************************************
***                                                                                                              ***
***                               BOOTVARE_V30.SPS                                              ***
***                                                                                                              ***
***   NOTE:  BOOTVARE version 2.2 and 2.3 were created in order to be          ***
***               compatible with the data from the Canadian Community Health     ***
***               Survey - cycle 1.2. and the National Population Health Survey -     ***
***               Health Institution component respectively.                                   *** 
***                                                                                                              ***
***               For a detailled documentation of this program, refer to version 2.1. ***
***                                                                                                              ***
*** This program calculates variance estimates using the bootstrap weights      ***
*** for different types of estimators.  Using SPSS macros, this program can     ***
*** calculate variance estimates for totals, ratios and differences between         ***
*** ratios.  It can also calculate variance estimates for the parameters              ***
*** of a regression and a logistic regression.   This program can also be            ***
*** customized for other types of analyses.                                                     ***
***                                                                                                               ***
*** The program is divided in 2 sections. Throughout the program, it is               ***
*** indicated where the user has to make changes.                                          ***
***                                                                                                               ***
*** Section 1: Declaration of the macro variables                                             ***
***            This is where variables that are going to be used throughout             ***
***            the program have to be defined.  Some changes must be made        ***
***            by the user.  This is also where the files containing the                    ***
***            variables to analyze and the bootstrap weights are read in.               ***
***                                                                                                               ***
*** Section 2: Declaration of the macros                                                         ***
***            This is where the user indicates what analysis they wish to              ***
***            perform.  Given the large number of observations to analyze,             ***
***            it is recommended that the number of analyses be limited,               ***
***            especially in the case of regressions.                                             ***
***                                                                                                               ***
*** This program calls the program MACROE_V30.SPS.  MACROE_V30.SPS  ***
*** contains different macros that enable variance estimation.  For standard       ***
*** use of the variance estimation program, no modification by the user to          ***
*** MACROE_V30.SPS is necessary.  Changes may be required in certain       ***
*** cases explained in section 2 of the program.                                               ***
***                                                                                                                ***
***                                                                                                                ***
*** Before using the program BOOTVARE_V30.SPS, the user has to prepare     ***
*** an analysis file that contains the variables to analyze.                                  ***
***                                                                                                                ***
*** To calculate a total, ratio or difference between ratios, the                            ***
*** estimates are obtained by summing the weights of the records that have       ***
*** the characteristic of interest.  Therefore, a dummy variable must be              ***
*** created for each of the variables to be analyzed.  This variable takes a          ***
*** value of 1 when the record has the characteristic and 0 otherwise.                ***
***                                                                                                               ***
*** NOTE: This file should only contain the records necessary for the                ***
*** analysis (ex: if the analysis is performed on the age group 12 and over,        ***
*** only the records for individuals 12 and over should be included in                 ***
*** the file.)                                                                                                  ***
**********************************************************************************************.


* All temporary files are written on C:\Temp and each file name has "SPSS_" as a
* prefix. 'C:\Temp\SPSS_All_Tots.sav' contains the Totals, Ratios, and Difference of Ratios Saved
* Parameters File and 'C:\Temp\SPSS_All_Regs.sav' contains the Regressions Saved Parameters File.
* You can delete all "SPSS_" files from C:\Temp after execution. You can also move files like the
* two previously mentioned for further use.

* Results can be found at the very end of the output, in the one or two "Text Output". Do not print
* the whole output which can be very large! Use the "Draft Viewer" if you run into memory problems.

*      Note:  Due to memory limitation issues,some problems may occur when performing linear or    
*                logistic regression analysis with SPSS version 10.                                                       


*********************************************************************************
***                               SECTION 1                                              ***
*********************************************************************************
***                                                                                              ***
***   This section lets the user specify the different parameters of     ***
***   interest (variable names, directory names, file names, etc.)      ***
***                                                                                              ***
*********************************************************************************.

DEFINE !Bootvar()

PRESERVE.
SET COM NO HEA ON MES ON ERR ON RES ON JOU ON LEN NONE WID 132 PRI NO MPR NO MITERATE=3000.

!Let !Tot=!Null
!Let !Reg=!Null



************************************************************************************************************
* SPECIFY THE ANALYSIS DATA FILE TO BE USED (created in step 1) (Folder and file) :   *
************************************************************************************************************.

!LET !Mfile='Folder_and_file_name_of_the_analysis_file_(step 1)'                                       /*  <- (ex: c:\data\analysis.sav). 



***********************************************************************************************
*   SPECIFY THE BOOTSTRAP WEIGHTS FOLDER AND FILE NAMES :           *
*               NB:  Only run one of the two following series of commands                 *
*               (comment the other one out, or erase it):                                           *
***********************************************************************************************.

* EXECUTE THIS PART IF THE BOOTSTRAP WEIGHTS ARE IN SPSS FORMAT (remove the " * ") *
*********************************************************************************************************************.


*!LET !BWsav=' Folder_and_file_names_of_the_bootstrap_weights_in_SPSS_format_(with_extension) '


* EXECUTE THIS PART IF THE BOOTSTRAP WEIGHTS ARE IN ASCII (.TXT) FORMAT (remove the " * ") *
****************************************************************************************************************************.

*!LET !BWTXT=' Folder_and_file_names_of_the_bootstrap_weights_in_text_format_(with_extension) '    
*!LET !Bwlay=' Folder_and_file_names_of_the_bootstrap_layout_file_in_SPSS_syntax_(with_extension) '
*!LET !Bwsav=' Folder_and_file_names_of_the_bootstrap_weights_in_SPSS_format_(created_in_this_section)_with_extension '

*FILE HANDLE BOOTLAY/NAME = !QUOTE(!BWTXT)  LRECL=13000.
*INCLUDE FILE=!QUOTE(!BWlay). 
*SAVE OUT=!QUOTE(!BWsav) /UNC.




****************************************************************************************************************************************
* SPECIFY, IF DESIRED, THE BREAKDOWN VARIABLE(S) (EG: PROVINCE, SEX, ETC...):                                  *
*       Write the name of the breakdown variable(s) below.                                                                                        *
*                                                                                                                                                                        *
*       -  If the analysis includes all of the data in the file created in step 1, put two single quotes (%LET !Classes = ' ' )*  
*       -  If more than one variable, leave a space between each variable  ( %LET !Classes = ' var1 var2 ' )                     *
*       -  DO NOT ERASE OR COMMENT OUT THIS COMMAND                                                                             *
****************************************************************************************************************************************.

!LET !Classes = ' breakdown_variable(s)_or_empty_(keep_the_2_single_quotes) '


********************************************************************************************************************************
* SPECIFY THE FOLLOWING INFORMATION (SPECIFIC TO THE SURVEY YOU ARE USING):                    *
*    You must specify:                                                                                                                                *
*           1- The unique identifier variable(s) (within quotes, separated by a space)                                            *
*           2- The Final Weight (variable included in the bootstrap weight file)                                                      *
*           3- The prefix of the boostraps weight variables                                                                                  *
*           4- The number of bootstrap weights to use (note: For testing, B must be >= 2.                                   *
*              IT IS NECESSARY TO USE ALL THE BOOTSTRAP WEIGHTS WHEN PERFORMING THE FINAL  *
*              ANALYSIS.  THE COMPLETE BOOTVARE_V30.SPS PROGRAM MUST THEN BE RUN.)              *
*                                                                                                                                                               *
*           - Refer to AppendixC_XYZ  to obtain this information                                                                         *
*********************************************************************************************************************************.

!LET !ident   = ' unique identifier variable(s) '      
!LET !fwgt    = final_weight
!LET !prefix  = prefix_of_bootstrap_weight_variables      
!LET !B       = number_of_weights_to_use 




***************************************************************************************************************************
* SPECIFY THE DIRECTORY AND THE NAME OF THE FILE THAT CONTAINS THE MACROS               *
* (THE PROGRAM MACROE_V30.SPS IF NO MODIFICATIONS HAVE BEEN MADE BY THE USER)     *
***************************************************************************************************************************.

INCLUDE FILE='directory_name_of_macroe_v30.sps\MACROE_V30.SPS'.      




* DO NOT MODIFY THE NEXT LINE. (GO TO SECTION 2)
********************************************************************.
!Prepare ident=!ident /fwgt=!fwgt /prefix=!prefix /B=!B /Classes=!Classes /mfile=!Mfile /bwsav=!bwsav.



************************************************************************************************
***                                      SECTION 2                                                          ***
************************************************************************************************
***                                                                                                                 ***
***          This section lets the user specify the different analyses of interest.        ***
***                                                                                                                 ***
************************************************************************************************.


*****************************************************************************************************
*** Variance estimates are calculated using the SPSS macros defined in the            ***
*** MACROE_V30.SPS program.  These SPSS macros can be submitted in this      ***  
*** program, to suit the user's needs.                                                                     ***
***                                                                                                                       ***
*** To submit a macro, the statement must specify the macro name and parameters  ***
*** to use.  In this program, the parameters indicate which variables will be                ***
*** used.                                                                                                              ***
***                                                                                                                       ***
*** Each macro submission gives an estimate of the variance of only one                  ***
*** parameter. If more than one parameter and variance need to be                           ***
*** calculated, the macro then must be submitted several times.                              ***
***                                                                                                                       ***
*** A statement for each possible type of analysis appears in the program. They        ***
*** are commented out and the user only needs to run those that are desired.            ***
***                                                                                                                       ***
*** Parameters "!B !Classes/" must be supplied exactly AS IS and BEFORE any       ***
*** Named Parameter                                                                                            ***
***                                                                                                                       ***
*** COMMENT ABOUT DIFFERENCES BETWEEN RATIOS:                                   ***
***  ----------------------------------------------------------------------------------                                   ***
***                                                                                                                       ***
*** The confidence interval is calculated for a single comparison of ratios.  If multiple  ***
*** comparisons are made, the method used to calculate the confidence interval        ***
*** must take this into account.  For this reason,  in the case of multiple                   ***
*** comparisons, the Z value from the normal distribution used in the calculation of     ***
*** the confidence interval must be corrected in the Result macro in the                     ***
*** MACROE_V30.SPS program using, for example, the Bonferroni approach for        ***
*** multiple comparisons.                                                                                      ***
***                                                                                                                       ***
*****************************************************************************************************.



* TO OBTAIN VARIANCE ESTIMATES OF A TOTAL, RUN:
*********************************************************************.

*!Total !B !Classes/ Var = name_of_the_variable .
*!Let !Tot=1.


* TO OBTAIN VARIANCE ESTIMATES OF A RATIO, RUN:
********************************************************************.

*!Ratio !B !Classes/ Varlist = name_of_numerator   name_of_denominator .
*!Let !Tot=1.


* TO OBTAIN VARIANCE ESTIMATES OF A DIFFERENCE OF RATIO, RUN:
******************************************************************************************.
          ****     NOTE: see the comment at the beginning of section 2 ...   ****

*!Diffrat !B !Classes/ Varlist =   variable1   variable2   variable3   variable4  .
*!Let !Tot=1.

                               *  where:   variable1   : the numerator variable of the first ratio          *   
                               *              variable2   : the denominator variable of the first ratio       *
                               *              variable3   : the numerator variable of the second ratio     *    
                               *              variable4   : the denominator variable of the second ratio  *.


* TO OBTAIN VARIANCE ESTIMATES OF REGRESSION PARAMETERS, RUN:
**********************************************************************************************.

*!Regress  !B !Classes/    Dep = dependant_variable  Indep = Independant_variable(s)
*!Let !Reg=1.


* TO OBTAIN VARIANCE ESTIMATES OF LOGISTIC REGRESSION PARAMETERS, RUN:
**********************************************************************************************************.
*!Log_Reg !B !Classes/ Cri=SAS Dep = dependant_variable  Indep = Independant_variable(s)
*!Let !Reg=1.

           * NOTE: Default iteration criteria differs between SAS and SPSS. 
           *       Use Cri=SAS to use SAS default values.
           *       Indep= must be specified LAST because of the unknown number of independent variables.



*********************************************************************
*        DO NOT MODIFY THE REMAINING LINES             *
*********************************************************************.

SET PRI NO MPR NO.

**** Printing of results ****.
!IF (!Tot=1) !THEN
+ !Print_T !Classes.   /* Printing of Totals, Ratios and Differences of Ratios results
!IFEND
!IF (!Reg=1) !THEN
+ !Print_R !Classes.   /* Printing of Linear and Logistic Regressions results
!IFEND
*****************************.

!Stop !NBlocks.

RESTORE.

!ENDDEFINE /* !Bootvar.

SET MPR ON.
!Bootvar.
SET MPR NO.

*** END OF BOOTVARE_V30.SPS PROGRAM  ***.
